"""The InfimaClient is used to interact with the infima platform."""

from typing import Dict

import httpx
import semver

from .client import Client
from .cohort import cohort
from .pool import pool
from .prediction import prediction
from .search import search
from .version import __version__

PeriodConvention = 0
FactorDateConvention = 1


class InfimaClient(Client):
    def __init__(
        self,
        token: str = None,
        base_url: str = "https://api.infima.io",
        extra_headers: Dict[str, str] = None,
        with_check=True,
    ):
        super(InfimaClient, self).__init__(base_url=base_url)
        if token:
            self.headers["INFIMA-TOKEN"] = token
        if extra_headers:
            self.headers.update(extra_headers)

        self.cohort = cohort(self)

        self.pool = pool(self)

        self.prediction = prediction(self)

        self.search = search(self)

        if with_check:
            assert self.check()

    def check(self) -> bool:
        """ Check if authentication is successful and for possible version upgrade."""
        try:
            with httpx.Client() as client:
                r = client.get(f"{self.base_url}/info")
                available = semver.VersionInfo.parse(r.json()["version"])
                current = semver.VersionInfo.parse(__version__)
                if available.major > current.major or (
                    available.major == current.major and available.minor > current.minor
                ):
                    print(
                        f"You are not using the most recent version of the client: {__version__}. Available: {available.major}.{available.minor}.0."
                    )
                else:
                    print(
                        f"You are using the most recent version of the client: {__version__}."
                    )

        except Exception as e:
            print(f"couldn't check version {e}")

        with httpx.Client(headers=self.headers) as client:
            r = client.get(f"{self.base_url}/api/v1/welcome")
            if r.status_code == 401:
                print("Your token doesn't seem valid. Please update it.")
                return False
            print("You are successfully authenticated.")
            return r.status_code == 200
        return False
