from typing import Any, Dict, List, Optional, Union

import attr

import infima_client.api.prediction_service.prediction_service_get_cohort as prediction_service_get_cohort
import infima_client.api.prediction_service.prediction_service_get_cohort_look_ahead as prediction_service_get_cohort_look_ahead
import infima_client.api.prediction_service.prediction_service_get_pools as prediction_service_get_pools
import infima_client.api.prediction_service.prediction_service_get_pools_look_ahead as prediction_service_get_pools_look_ahead
import infima_client.api.prediction_service.prediction_service_get_portfolio as prediction_service_get_portfolio
import infima_client.api.prediction_service.prediction_service_get_portfolio_look_ahead as prediction_service_get_portfolio_look_ahead
from infima_client.models import *

from .client import Client
from .types import UNSET, Response, Unset
from .util import named_arguments_only


class prediction:
    def __init__(self, client: Client):
        self.client = client
        self._detailed = predictionDetailed(client)

    @named_arguments_only
    def get_cohort(
        self,
        cohort: str,
        as_ofyear: Union[Unset, int] = UNSET,
        as_ofmonth: Union[Unset, int] = UNSET,
        as_ofday: Union[Unset, int] = UNSET,
        convention: Union[
            Unset, PredictionService_GetCohortConvention
        ] = PredictionService_GetCohortConvention.PERIOD,
        scenario: Union[
            Unset, PredictionService_GetCohortScenario
        ] = PredictionService_GetCohortScenario.BASELINE,
    ) -> Optional[
        Union[
            CohortPrediction,
            CohortPredictionBadRequest,
            CohortPredictionUnauthorized,
            CohortPredictionNotFound,
            CohortPredictionError,
        ]
    ]:

        """
        Fetch prepayment prediction for a single cohort for an as-of date, default to today.

        Args:
            cohort: Cohort name to fetch aggregated prepayment prediction.
            as_ofyear: Year of the date. Must be from 1 to 9999, or 0 to specify a date without a year
            as_ofmonth: Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
            as_ofday: Day of a month. Must be from 1 to 31 and valid for the year and month, or 0 to specify a year by itself or a year and month where the day isn't significant.Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
            convention: Convention to use for the date, can be PERIOD or FACTOR_DATE.
            scenario: None

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return prediction_service_get_cohort.sync(
            client=self.client,
            cohort=cohort,
            as_ofyear=as_ofyear,
            as_ofmonth=as_ofmonth,
            as_ofday=as_ofday,
            convention=convention,
            scenario=scenario,
        )

    @named_arguments_only
    def get_cohort_look_ahead(
        self,
        cohort: str,
        fromyear: Union[Unset, int] = UNSET,
        frommonth: Union[Unset, int] = UNSET,
        fromday: Union[Unset, int] = UNSET,
        convention: Union[
            Unset, PredictionService_GetCohortLookAheadConvention
        ] = PredictionService_GetCohortLookAheadConvention.PERIOD,
        scenario: Union[
            Unset, PredictionService_GetCohortLookAheadScenario
        ] = PredictionService_GetCohortLookAheadScenario.BASELINE,
        ahead: Union[
            Unset, PredictionService_GetCohortLookAheadAhead
        ] = PredictionService_GetCohortLookAheadAhead.NONE,
    ) -> Optional[
        Union[
            CohortLookAheadPrediction,
            CohortLookAheadPredictionBadRequest,
            CohortLookAheadPredictionUnauthorized,
            CohortLookAheadPredictionNotFound,
            CohortLookAheadPredictionError,
        ]
    ]:

        """
        Fetch historical cohort prepayment predictions for a given look-ahead.

        Args:
            cohort: Cohort name to fetch aggregated historical look ahead prepayment prediction.
            fromyear: Year of the date. Must be from 1 to 9999, or 0 to specify a date without a year
            frommonth: Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
            fromday: Day of a month. Must be from 1 to 31 and valid for the year and month, or 0 to specify a year by itself or a year and month where the day isn't significant.Month of a year. Must be from 1 to 12, or 0 to specify a year without a month and day.
            convention: Convention to use for the date, can be PERIOD or FACTOR_DATE.
            scenario: None
            ahead: None

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return prediction_service_get_cohort_look_ahead.sync(
            client=self.client,
            cohort=cohort,
            fromyear=fromyear,
            frommonth=frommonth,
            fromday=fromday,
            convention=convention,
            scenario=scenario,
            ahead=ahead,
        )

    @named_arguments_only
    def get_pools(
        self,
        as_of: Union[PoolPredictionAsOf, Unset] = UNSET,
        convention: Union[
            Unset, PredictionService_GetPoolsJsonBodyConvention
        ] = PredictionService_GetPoolsJsonBodyConvention.PERIOD,
        cusips: Union[Unset, List[str]] = UNSET,
        scenario: Union[
            Unset, PredictionService_GetPoolsJsonBodyScenario
        ] = PredictionService_GetPoolsJsonBodyScenario.BASELINE,
    ) -> Optional[
        Union[
            PoolPredictions,
            PoolPredictionsBadRequest,
            PoolPredictionsUnauthorized,
            PoolPredictionsNotFound,
            PoolPredictionsError,
        ]
    ]:

        """
        Fetch individual pool prepayment predictions for an as-of date, default to today.

        Args:

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return prediction_service_get_pools.sync(
            client=self.client,
            json_body=PoolPredictionRequest(
                as_of=as_of, convention=convention, cusips=cusips, scenario=scenario
            ),
        )

    @named_arguments_only
    def get_pools_look_ahead(
        self,
        ahead: Union[
            Unset, PredictionService_GetPoolsLookAheadJsonBodyAhead
        ] = PredictionService_GetPoolsLookAheadJsonBodyAhead.NONE,
        convention: Union[
            Unset, PredictionService_GetPoolsLookAheadJsonBodyConvention
        ] = PredictionService_GetPoolsLookAheadJsonBodyConvention.PERIOD,
        cusips: Union[Unset, List[str]] = UNSET,
        from_: Union[PredictionService_GetPoolsLookAheadJsonBodyFrom, Unset] = UNSET,
        scenario: Union[
            Unset, PredictionService_GetPoolsLookAheadJsonBodyScenario
        ] = PredictionService_GetPoolsLookAheadJsonBodyScenario.BASELINE,
    ) -> Optional[
        Union[
            PoolLookAheadPredictions,
            PoolLookAheadPredictionsBadRequest,
            PoolLookAheadPredictionsUnauthorized,
            PoolLookAheadPredictionsNotFound,
            PoolLookAheadPredictionsError,
        ]
    ]:

        """
        Fetch individual historical pool prepayment predictions for a given look-ahead.

        Args:

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return prediction_service_get_pools_look_ahead.sync(
            client=self.client,
            json_body=PoolLookAheadPredictionRequest(
                ahead=ahead,
                convention=convention,
                cusips=cusips,
                from_=from_,
                scenario=scenario,
            ),
        )

    @named_arguments_only
    def get_portfolio(
        self,
        as_of: Union[PortfolioPredictionAsOf, Unset] = UNSET,
        convention: Union[
            Unset, PredictionService_GetPortfolioJsonBodyConvention
        ] = PredictionService_GetPortfolioJsonBodyConvention.PERIOD,
        cusips: Union[Unset, List[str]] = UNSET,
        scenario: Union[
            Unset, PredictionService_GetPortfolioJsonBodyScenario
        ] = PredictionService_GetPortfolioJsonBodyScenario.BASELINE,
    ) -> Optional[
        Union[
            PortfolioPrediction,
            PortfolioPredictionBadRequest,
            PortfolioPredictionUnauthorized,
            PortfolioPredictionNotFound,
            PortfolioPredictionError,
        ]
    ]:

        """
        Weighted aggregate of the prepayment prediction of the portfolio components for an as-of date, default to today.

        Args:

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return prediction_service_get_portfolio.sync(
            client=self.client,
            json_body=PortfolioPredictionRequest(
                as_of=as_of, convention=convention, cusips=cusips, scenario=scenario
            ),
        )

    @named_arguments_only
    def get_portfolio_look_ahead(
        self,
        ahead: Union[
            Unset, PredictionService_GetPortfolioLookAheadJsonBodyAhead
        ] = PredictionService_GetPortfolioLookAheadJsonBodyAhead.NONE,
        convention: Union[
            Unset, PredictionService_GetPortfolioLookAheadJsonBodyConvention
        ] = PredictionService_GetPortfolioLookAheadJsonBodyConvention.PERIOD,
        cusips: Union[Unset, List[str]] = UNSET,
        from_: Union[
            PredictionService_GetPortfolioLookAheadJsonBodyFrom, Unset
        ] = UNSET,
        scenario: Union[
            Unset, PredictionService_GetPortfolioLookAheadJsonBodyScenario
        ] = PredictionService_GetPortfolioLookAheadJsonBodyScenario.BASELINE,
    ) -> Optional[
        Union[
            PortfolioLookAheadPrediction,
            PortfolioLookAheadPredictionBadRequest,
            PortfolioLookAheadPredictionUnauthorized,
            PortfolioLookAheadPredictionNotFound,
            PortfolioLookAheadPredictionError,
        ]
    ]:

        """
        Weighted aggregate of the historical prepayment look-ahead prediction of the portfolio components.

        Args:

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return prediction_service_get_portfolio_look_ahead.sync(
            client=self.client,
            json_body=PortfolioLookAheadPredictionRequest(
                ahead=ahead,
                convention=convention,
                cusips=cusips,
                from_=from_,
                scenario=scenario,
            ),
        )


class predictionDetailed:
    def __init__(self, client: Client):
        self.client = client

    @named_arguments_only
    def get_cohort(
        self,
        cohort: str,
        as_ofyear: Union[Unset, int] = UNSET,
        as_ofmonth: Union[Unset, int] = UNSET,
        as_ofday: Union[Unset, int] = UNSET,
        convention: Union[
            Unset, PredictionService_GetCohortConvention
        ] = PredictionService_GetCohortConvention.PERIOD,
        scenario: Union[
            Unset, PredictionService_GetCohortScenario
        ] = PredictionService_GetCohortScenario.BASELINE,
    ) -> Response[
        Union[
            CohortPrediction,
            CohortPredictionBadRequest,
            CohortPredictionUnauthorized,
            CohortPredictionNotFound,
            CohortPredictionError,
        ]
    ]:

        return prediction_service_get_cohort.sync_detailed(
            client=self.client,
            cohort=cohort,
            as_ofyear=as_ofyear,
            as_ofmonth=as_ofmonth,
            as_ofday=as_ofday,
            convention=convention,
            scenario=scenario,
        )

    @named_arguments_only
    def get_cohort_look_ahead(
        self,
        cohort: str,
        fromyear: Union[Unset, int] = UNSET,
        frommonth: Union[Unset, int] = UNSET,
        fromday: Union[Unset, int] = UNSET,
        convention: Union[
            Unset, PredictionService_GetCohortLookAheadConvention
        ] = PredictionService_GetCohortLookAheadConvention.PERIOD,
        scenario: Union[
            Unset, PredictionService_GetCohortLookAheadScenario
        ] = PredictionService_GetCohortLookAheadScenario.BASELINE,
        ahead: Union[
            Unset, PredictionService_GetCohortLookAheadAhead
        ] = PredictionService_GetCohortLookAheadAhead.NONE,
    ) -> Response[
        Union[
            CohortLookAheadPrediction,
            CohortLookAheadPredictionBadRequest,
            CohortLookAheadPredictionUnauthorized,
            CohortLookAheadPredictionNotFound,
            CohortLookAheadPredictionError,
        ]
    ]:

        return prediction_service_get_cohort_look_ahead.sync_detailed(
            client=self.client,
            cohort=cohort,
            fromyear=fromyear,
            frommonth=frommonth,
            fromday=fromday,
            convention=convention,
            scenario=scenario,
            ahead=ahead,
        )

    @named_arguments_only
    def get_pools(
        self,
        as_of: Union[PoolPredictionAsOf, Unset] = UNSET,
        convention: Union[
            Unset, PredictionService_GetPoolsJsonBodyConvention
        ] = PredictionService_GetPoolsJsonBodyConvention.PERIOD,
        cusips: Union[Unset, List[str]] = UNSET,
        scenario: Union[
            Unset, PredictionService_GetPoolsJsonBodyScenario
        ] = PredictionService_GetPoolsJsonBodyScenario.BASELINE,
    ) -> Response[
        Union[
            PoolPredictions,
            PoolPredictionsBadRequest,
            PoolPredictionsUnauthorized,
            PoolPredictionsNotFound,
            PoolPredictionsError,
        ]
    ]:

        return prediction_service_get_pools.sync_detailed(
            client=self.client,
            json_body=PoolPredictionRequest(
                as_of=as_of, convention=convention, cusips=cusips, scenario=scenario
            ),
        )

    @named_arguments_only
    def get_pools_look_ahead(
        self,
        ahead: Union[
            Unset, PredictionService_GetPoolsLookAheadJsonBodyAhead
        ] = PredictionService_GetPoolsLookAheadJsonBodyAhead.NONE,
        convention: Union[
            Unset, PredictionService_GetPoolsLookAheadJsonBodyConvention
        ] = PredictionService_GetPoolsLookAheadJsonBodyConvention.PERIOD,
        cusips: Union[Unset, List[str]] = UNSET,
        from_: Union[PredictionService_GetPoolsLookAheadJsonBodyFrom, Unset] = UNSET,
        scenario: Union[
            Unset, PredictionService_GetPoolsLookAheadJsonBodyScenario
        ] = PredictionService_GetPoolsLookAheadJsonBodyScenario.BASELINE,
    ) -> Response[
        Union[
            PoolLookAheadPredictions,
            PoolLookAheadPredictionsBadRequest,
            PoolLookAheadPredictionsUnauthorized,
            PoolLookAheadPredictionsNotFound,
            PoolLookAheadPredictionsError,
        ]
    ]:

        return prediction_service_get_pools_look_ahead.sync_detailed(
            client=self.client,
            json_body=PoolLookAheadPredictionRequest(
                ahead=ahead,
                convention=convention,
                cusips=cusips,
                from_=from_,
                scenario=scenario,
            ),
        )

    @named_arguments_only
    def get_portfolio(
        self,
        as_of: Union[PortfolioPredictionAsOf, Unset] = UNSET,
        convention: Union[
            Unset, PredictionService_GetPortfolioJsonBodyConvention
        ] = PredictionService_GetPortfolioJsonBodyConvention.PERIOD,
        cusips: Union[Unset, List[str]] = UNSET,
        scenario: Union[
            Unset, PredictionService_GetPortfolioJsonBodyScenario
        ] = PredictionService_GetPortfolioJsonBodyScenario.BASELINE,
    ) -> Response[
        Union[
            PortfolioPrediction,
            PortfolioPredictionBadRequest,
            PortfolioPredictionUnauthorized,
            PortfolioPredictionNotFound,
            PortfolioPredictionError,
        ]
    ]:

        return prediction_service_get_portfolio.sync_detailed(
            client=self.client,
            json_body=PortfolioPredictionRequest(
                as_of=as_of, convention=convention, cusips=cusips, scenario=scenario
            ),
        )

    @named_arguments_only
    def get_portfolio_look_ahead(
        self,
        ahead: Union[
            Unset, PredictionService_GetPortfolioLookAheadJsonBodyAhead
        ] = PredictionService_GetPortfolioLookAheadJsonBodyAhead.NONE,
        convention: Union[
            Unset, PredictionService_GetPortfolioLookAheadJsonBodyConvention
        ] = PredictionService_GetPortfolioLookAheadJsonBodyConvention.PERIOD,
        cusips: Union[Unset, List[str]] = UNSET,
        from_: Union[
            PredictionService_GetPortfolioLookAheadJsonBodyFrom, Unset
        ] = UNSET,
        scenario: Union[
            Unset, PredictionService_GetPortfolioLookAheadJsonBodyScenario
        ] = PredictionService_GetPortfolioLookAheadJsonBodyScenario.BASELINE,
    ) -> Response[
        Union[
            PortfolioLookAheadPrediction,
            PortfolioLookAheadPredictionBadRequest,
            PortfolioLookAheadPredictionUnauthorized,
            PortfolioLookAheadPredictionNotFound,
            PortfolioLookAheadPredictionError,
        ]
    ]:

        return prediction_service_get_portfolio_look_ahead.sync_detailed(
            client=self.client,
            json_body=PortfolioLookAheadPredictionRequest(
                ahead=ahead,
                convention=convention,
                cusips=cusips,
                from_=from_,
                scenario=scenario,
            ),
        )
